@@ -102,55 +102,55 @@ class BaseTestNodes(object):
                 self.elem3_second, self.doc).ancestors[-1].is_document)
 
     def test_children_attribute(self):
-        self.assertEquals(self.elem1, self.xml4h_root.children[0].impl_node)
-        self.assertEquals([u'元素1', 'Element2', 'Element3', 'Element4'],
+        self.assertEqual(self.elem1, self.xml4h_root.children[0].impl_node)
+        self.assertEqual([u'元素1', 'Element2', 'Element3', 'Element4'],
             [n.name for n in self.xml4h_root.children])
-        self.assertEquals(['Element2'],
+        self.assertEqual(['Element2'],
             [n.name for n in self.xml4h_root.find_first('Element3').children])
 
     def test_children_nodelist_callable(self):
         """Use callable feature of children attribute's returned NodeList"""
-        self.assertEquals(self.elem1, self.xml4h_root.children()[0].impl_node)
-        self.assertEquals([u'元素1', 'Element2', 'Element3', 'Element4'],
+        self.assertEqual(self.elem1, self.xml4h_root.children()[0].impl_node)
+        self.assertEqual([u'元素1', 'Element2', 'Element3', 'Element4'],
             [n.name for n in self.xml4h_root.children()])
-        self.assertEquals(['Element2'],
+        self.assertEqual(['Element2'],
             [n.name for n in
                 self.xml4h_root.find_first('Element3').children()])
         # Filter children by name
-        self.assertEquals(['Element2'],
+        self.assertEqual(['Element2'],
             [n.name for n in self.xml4h_root.children(name='Element2')])
         # Filter children by local name
-        self.assertEquals(['Element2'],
+        self.assertEqual(['Element2'],
             [n.name for n in self.xml4h_root.children(local_name='Element2')])
         # Filter children by namespace
-        self.assertEquals(['Element3', 'Element4'],
+        self.assertEqual(['Element3', 'Element4'],
             [n.name for n in self.xml4h_root.children(ns_uri='urn:ns1')])
         # Filter by node type
-        self.assertEquals([],
+        self.assertEqual([],
             [n.name for n in self.xml4h_root.children(
                                   node_type=xml4h.nodes.Text)])
-        self.assertEquals([u'元素1', 'Element2', 'Element3', 'Element4'],
+        self.assertEqual([u'元素1', 'Element2', 'Element3', 'Element4'],
             [n.name for n in self.xml4h_root.children(
                                   node_type=xml4h.nodes.ELEMENT_NODE)])
-        self.assertEquals([u'元素1', 'Element2', 'Element3', 'Element4'],
+        self.assertEqual([u'元素1', 'Element2', 'Element3', 'Element4'],
             [n.name for n in self.xml4h_root.children(
                                   node_type=xml4h.nodes.Element)])
         # Filter by arbitrary function
         fn = lambda x: x.name[-1] in ('1', '3')
-        self.assertEquals([u'元素1', 'Element3'],
+        self.assertEqual([u'元素1', 'Element3'],
             [n.name for n in self.xml4h_root.children(filter_fn=fn)])
         # Return first result only via `first_only` flag
-        self.assertEquals('Element3',
+        self.assertEqual('Element3',
             self.xml4h_root.children(ns_uri='urn:ns1', first_only=True).name)
         self.assertEqual(None,
             self.xml4h_root.children(ns_uri='urn:wrong', first_only=True))
         # Return first result only via `first` attribute on children
-        self.assertEquals('Element3',
+        self.assertEqual('Element3',
             self.xml4h_root.children(ns_uri='urn:ns1').first.name)
         self.assertEqual(None,
             self.xml4h_root.children(ns_uri='urn:wrong').first)
         # Return first result only via `child` method
-        self.assertEquals('Element3',
+        self.assertEqual('Element3',
             self.xml4h_root.child(ns_uri='urn:ns1').name)
         self.assertEqual(None,
             self.xml4h_root.child(ns_uri='urn:wrong'))
@@ -248,11 +248,19 @@ class BaseTestNodes(object):
         # Get element's attributes dict
         wrapped_elem = self.adapter_class.wrap_node(self.elem1, self.doc)
         attrs_dict = wrapped_elem.attributes
-        self.assertEqual(['a', 'ns1:b', 'xmlns:ns1'], attrs_dict.keys())
-        self.assertEqual(['1', '2', 'urn:ns1'], attrs_dict.values())
+        self.assertEqual(
+            sorted(['a', 'ns1:b', 'xmlns:ns1']),
+            sorted(attrs_dict.keys()))
+        self.assertEqual(
+            ['1', '2', 'urn:ns1'],
+            [attrs_dict['a'], attrs_dict['ns1:b'], attrs_dict['xmlns:ns1']])
         # Set/change attributes via element methods
         wrapped_elem.set_attributes({'a': 10, 'c': 3})
-        self.assertEqual(['10', '3', '2', 'urn:ns1'],
+        self.assertEqual(
+            sorted(['a', 'c', 'ns1:b', 'xmlns:ns1']),
+            sorted(attrs_dict.keys()))
+        self.assertEqual(
+            ['10', '3', '2', 'urn:ns1'],
             [a.value for a in wrapped_elem.attribute_nodes])
         # Set attributes via dict assignment (pre-existing attrs are deleted)
         wrapped_elem.attributes = {
@@ -269,8 +277,9 @@ class BaseTestNodes(object):
                 'xmlns:ns2'],
                 [a.name for a in wrapped_elem.attribute_nodes])
         else:
-            self.assertEqual(['a', 'ns2:d', 'ns2:x', 'xmlns:ns2'],
-                [a.name for a in wrapped_elem.attribute_nodes])
+            self.assertEqual(
+                sorted(['a', 'ns2:d', 'ns2:x', 'xmlns:ns2']),
+                sorted([a.name for a in wrapped_elem.attribute_nodes]))
         self.assertEqual('100', wrapped_elem.attributes['a'])
         self.assertEqual('-3', wrapped_elem.attributes['ns2:d'])
         self.assertEqual('urn:ns2', wrapped_elem.attributes['xmlns:ns2'])
@@ -307,15 +316,27 @@ class BaseTestNodes(object):
         # Set text on element
         wrapped_node.text = 'Different text'
         self.assertEqual('Different text', wrapped_node.text)
-        self.assertEqual(
-            wrapped_node.children[0].value,
-            self.adapter_class(self.doc).get_node_value(
-                wrapped_node.children[0].impl_node))
         # Unset text on element (removes any text children)
         wrapped_node.text = None
         self.assertEqual(None, wrapped_node.text)
         self.assertEqual([], wrapped_node.children)
 
+    def test_element_value(self):
+        # Get value on element
+        wrapped_node = self.adapter_class.wrap_node(self.elem1, self.doc)
+        self.assertEqual('Some text', wrapped_node.text)
+        # Only Minidom library distinguishes between node's text & value
+        if isinstance(self, TestMinidomNodes):
+            self.assertIsNone(wrapped_node.value)
+        else:
+            self.assertEqual('Some text', wrapped_node.value)
+        # Set value on element
+        wrapped_node.value = 'Different text'
+        self.assertEqual('Different text', wrapped_node.value)
+        # Unset value on element
+        wrapped_node.value = None
+        self.assertEqual(None, wrapped_node.value)
+
     def test_delete(self):
         # Remove single Element using delete() method without destroying it
         self.assertEqual(
@@ -530,7 +551,7 @@ class BaseTestNodes(object):
         try:
             self.xml4h_root.DoesNotExist
             self.fail('Expected AttributeError')
-        except AttributeError, e:
+        except AttributeError as e:
             self.assertEqual(
                 """<xml4h.nodes.Element: "DocRoot"> object"""
                 """ has no attribute 'DoesNotExist'""",
@@ -538,7 +559,7 @@ class BaseTestNodes(object):
         # Look up non-existent attribute by Python key name
         try:
             self.xml4h_root['not-an-attribute']
-        except KeyError, e:
+        except KeyError as e:
             self.assertEqual("'not-an-attribute'", str(e))
 
         # Look up child element by attribute
@@ -575,12 +596,12 @@ class BaseTestNodes(object):
         try:
             self.xml4h_root._leadingunderscore
             self.fail('Expected AttributeError')
-        except AttributeError, e:
+        except AttributeError as e:
             pass
         try:
             self.xml4h_root._leadingunderscore_
             self.fail('Expected AttributeError')
-        except AttributeError, e:
+        except AttributeError as e:
             pass
 
         # Look up an attribute by key name
