@@ -1,11 +1,7 @@
 # -*- coding: utf-8 -*-
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
-import sys
+import unittest
 import functools
-from StringIO import StringIO
+import six
 
 import xml4h
 
@@ -23,10 +19,10 @@ class BaseWriterTest(object):
                 .element('Elem1').text(u'默认جذ').up()
                 .element('Elem2'))
         # Handy IO writer
-        self.iostr = StringIO()
+        self.iobytes = six.BytesIO()
 
-    def test_write_to_stdout_by_default(self):
-        # Default write output is utf-8, with no pretty-printing
+    def test_write_defaults(self):
+        """ Default write output is utf-8 with no pretty-printing """
         xml = (
             u'<?xml version="1.0" encoding="utf-8"?>'
             u'<DocRoot>'
@@ -34,17 +30,16 @@ class BaseWriterTest(object):
             u'<Elem2/>'
             u'</DocRoot>'
             )
-        try:
-            old_stdout = sys.stdout
-            sys.stdout = StringIO()
-            self.builder.write_doc()
-            self.assertEqual(xml.encode('utf-8'), sys.stdout.getvalue())
-        finally:
-            sys.stdout = old_stdout
+        io_string = six.StringIO()
+        self.builder.write_doc(io_string)
+        if six.PY2:
+            self.assertEqual(xml.encode('utf-8'), io_string.getvalue())
+        else:
+            self.assertEqual(xml, io_string.getvalue())
 
     def test_write_current_node_and_descendents(self):
-        self.builder.dom_element.write(self.iostr)
-        self.assertEqual('<Elem2/>', self.iostr.getvalue())
+        self.builder.dom_element.write(self.iobytes)
+        self.assertEqual(b'<Elem2/>', self.iobytes.getvalue())
 
     def test_write_utf8_by_default(self):
         # Default write output is utf-8, with no pretty-printing
@@ -55,8 +50,8 @@ class BaseWriterTest(object):
             u'<Elem2/>'
             u'</DocRoot>'
             )
-        self.builder.dom_element.write_doc(self.iostr)
-        self.assertEqual(xml.encode('utf-8'), self.iostr.getvalue())
+        self.builder.dom_element.write_doc(self.iobytes)
+        self.assertEqual(xml.encode('utf-8'), self.iobytes.getvalue())
 
     def test_write_utf16(self):
         xml = (
@@ -66,13 +61,13 @@ class BaseWriterTest(object):
             u'<Elem2/>'
             u'</DocRoot>'
             )
-        self.builder.dom_element.write_doc(self.iostr, encoding='utf-16')
-        self.assertEqual(xml.encode('utf-16'), self.iostr.getvalue())
+        self.builder.dom_element.write_doc(self.iobytes, encoding='utf-16')
+        self.assertEqual(xml.encode('utf-16'), self.iobytes.getvalue())
 
     def test_write_latin1_with_illegal_characters(self):
         self.assertRaises(UnicodeEncodeError,
             self.builder.dom_element.write_doc,
-                self.iostr, encoding='latin1', indent=2)
+                self.iobytes, encoding='latin1', indent=2)
 
     def test_write_latin1(self):
         # Create latin1-friendly test document
@@ -80,14 +75,14 @@ class BaseWriterTest(object):
             self.my_builder('DocRoot')
                 .element('Elem1').text(u'Tést çæsè').up()
                 .element('Elem2'))
-        self.builder.dom_element.write_doc(self.iostr, encoding='latin1')
+        self.builder.dom_element.write_doc(self.iobytes, encoding='latin1')
         self.assertEqual(
             u'<?xml version="1.0" encoding="latin1"?>'
             u'<DocRoot>'
             u'<Elem1>Tést çæsè</Elem1>'
             u'<Elem2/>'
             u'</DocRoot>'.encode('latin1'),
-            self.iostr.getvalue())
+            self.iobytes.getvalue())
 
     def test_with_no_encoding(self):
         """No encoding writes python unicode"""
@@ -98,33 +93,34 @@ class BaseWriterTest(object):
             u'<Elem2/>'
             u'</DocRoot>'
             )
-        self.builder.dom_element.write_doc(self.iostr, encoding=None)
+        io_string = six.StringIO()
+        self.builder.dom_element.write_doc(io_string, encoding=None)
         # NOTE Exact test, no encoding of comparison XML doc string
-        self.assertEqual(xml, self.iostr.getvalue())
+        self.assertEqual(xml, io_string.getvalue())
 
     def test_omit_declaration(self):
-        self.builder.dom_element.write_doc(self.iostr,
+        self.builder.dom_element.write_doc(self.iobytes,
                 omit_declaration=True)
         self.assertEqual(
             u'<DocRoot>'
             u'<Elem1>默认جذ</Elem1>'
             u'<Elem2/>'
             u'</DocRoot>'.encode('utf-8'),
-            self.iostr.getvalue())
+            self.iobytes.getvalue())
 
     def test_default_indent_and_newline(self):
         """Default indent of 4 spaces with newlines when indent=True"""
-        self.builder.dom_element.write_doc(self.iostr, indent=True)
+        self.builder.dom_element.write_doc(self.iobytes, indent=True)
         self.assertEqual(
             u'<?xml version="1.0" encoding="utf-8"?>\n'
             u'<DocRoot>\n'
             u'    <Elem1>默认جذ</Elem1>\n'
             u'    <Elem2/>\n'
             u'</DocRoot>\n'.encode('utf-8'),
-            self.iostr.getvalue())
+            self.iobytes.getvalue())
 
     def test_custom_indent_and_newline(self):
-        self.builder.dom_element.write_doc(self.iostr,
+        self.builder.dom_element.write_doc(self.iobytes,
             indent=8, newline='\t')
         self.assertEqual(
             u'<?xml version="1.0" encoding="utf-8"?>\t'
@@ -132,7 +128,7 @@ class BaseWriterTest(object):
             u'        <Elem1>默认جذ</Elem1>\t'
             u'        <Elem2/>\t'
             u'</DocRoot>\t'.encode('utf-8'),
-            self.iostr.getvalue())
+            self.iobytes.getvalue())
 
 
 class TestXmlDomBuilder(BaseWriterTest, unittest.TestCase):
