@@ -35,6 +35,10 @@ class LXMLAdapter(XmlImplAdapter):
             cls.ignore_whitespace_text_nodes(wrapped_doc)
         return wrapped_doc
 
+    @classmethod
+    def parse_bytes(cls, xml_bytes, ignore_whitespace_text_nodes=True):
+        return LXMLAdapter.parse_string(xml_bytes, ignore_whitespace_text_nodes)
+
     @classmethod
     def parse_file(cls, xml_file, ignore_whitespace_text_nodes=True):
         impl_doc = etree.parse(xml_file)
@@ -89,7 +93,7 @@ class LXMLAdapter(XmlImplAdapter):
             # TODO This doesn't seem to help
             curr_node = parent
             while curr_node.__class__ == etree._Element:
-                for n, v in curr_node.attrib.items():
+                for n, v in list(curr_node.attrib.items()):
                     if '{%s}' % nodes.Node.XMLNS_URI in n:
                         _, prefix = n.split('}')
                         my_nsmap[prefix] = v
@@ -203,25 +207,49 @@ class LXMLAdapter(XmlImplAdapter):
             return '#comment'
         elif isinstance(node, etree._ProcessingInstruction):
             return node.target
-        elif self.get_node_name_prefix(node) is not None:
-            return '%s:%s' % (node.prefix, self.get_node_local_name(node))
+        prefix = self.get_node_name_prefix(node)
+        local_name = self.get_node_local_name(node)
+        if prefix is not None:
+            return '%s:%s' % (prefix, local_name)
         else:
-            return self.get_node_local_name(node)
+            return local_name
 
     def get_node_local_name(self, node):
         return re.sub('{.*}', '', node.tag)
 
     def get_node_name_prefix(self, node):
-        return node.prefix
+        # Believe non-Element nodes that have a prefix set (e.g. LXMLAttribute)
+        if node.prefix and not isinstance(node, etree._Element):
+            return node.prefix
+        # Derive prefix by unpacking node name
+        qname, ns_uri, prefix, local_name = self._unpack_name(node.tag, node)
+        if prefix:
+            # Don't add unnecessary excess namespace prefixes for elements
+            # with a local default namespace declaration
+            xmlns_val = self.get_node_attribute_value(node, 'xmlns')
+            if xmlns_val == ns_uri:
+                return None
+            # Don't add unnecessary excess namespace prefixes for default ns
+            if prefix == 'xmlns':
+                return None
+            else:
+                return prefix
+        else:
+            return None
 
     def get_node_value(self, node):
         if isinstance(node, (etree._ProcessingInstruction, etree._Comment)):
             return node.text
-        else:
+        elif hasattr(node, 'value'):
             return node.value
+        else:
+            return node.text
 
     def set_node_value(self, node, value):
-        node.value = value
+        if hasattr(node, 'value'):
+            node.value = value
+        else:
+            self.set_node_text(node, value)
 
     def get_node_text(self, node):
         return node.text
@@ -232,13 +260,13 @@ class LXMLAdapter(XmlImplAdapter):
     def get_node_attributes(self, element, ns_uri=None):
         # TODO: Filter by ns_uri
         attribs_by_qname = {}
-        for n, v in element.attrib.items():
+        for n, v in list(element.attrib.items()):
             qname, ns_uri, prefix, local_name = self._unpack_name(n, element)
             attribs_by_qname[qname] = LXMLAttribute(
                 qname, ns_uri, prefix, local_name, v, element)
         # Include namespace declarations, which we also treat as attributes
         if element.nsmap:
-            for n, v in element.nsmap.items():
+            for n, v in list(element.nsmap.items()):
                 # Only add namespace as attribute if not defined in ancestors
                 # and not the global xmlns namespace
                 if (self._is_ns_in_ancestor(element, n, v)
@@ -252,7 +280,7 @@ class LXMLAdapter(XmlImplAdapter):
                     ns_attr_name, element)
                 attribs_by_qname[qname] = LXMLAttribute(
                     qname, ns_uri, prefix, local_name, v, element)
-        return attribs_by_qname.values()
+        return list(attribs_by_qname.values())
 
     def has_node_attribute(self, element, name, ns_uri=None):
         return name in [a.qname for a
@@ -363,6 +391,7 @@ class LXMLAdapter(XmlImplAdapter):
             return child
 
     def lookup_ns_uri_by_attr_name(self, node, name):
+        ns_name = None
         if name == 'xmlns':
             ns_name = None
         elif name.startswith('xmlns:'):
@@ -384,8 +413,8 @@ class LXMLAdapter(XmlImplAdapter):
         if uri == nodes.Node.XMLNS_URI:
             return 'xmlns'
         result = None
-        if hasattr(node, 'nsmap') and uri in node.nsmap.values():
-            for n, v in node.nsmap.items():
+        if hasattr(node, 'nsmap') and uri in list(node.nsmap.values()):
+            for n, v in list(node.nsmap.items()):
                 if v == uri:
                     result = n
                     break
@@ -397,7 +426,7 @@ class LXMLAdapter(XmlImplAdapter):
             # lxml and we'd rather use a human-assigned prefix if available.
             curr_node = node  # self.get_node_parent(node)
             while curr_node.__class__ == etree._Element:
-                for n, v in curr_node.attrib.items():
+                for n, v in list(curr_node.attrib.items()):
                     if v == uri and ('{%s}' % nodes.Node.XMLNS_URI) in n:
                         result = n.split('}')[1]
                         return result
@@ -443,7 +472,7 @@ class LXMLAdapter(XmlImplAdapter):
             if (hasattr(curr_node, 'nsmap')
                     and curr_node.nsmap.get(name) == value):
                 return True
-            for n, v in curr_node.attrib.items():
+            for n, v in list(curr_node.attrib.items()):
                 if v == value and '{%s}' % nodes.Node.XMLNS_URI in n:
                     return True
             curr_node = self.get_node_parent(curr_node)
