@@ -1,7 +1,7 @@
 import re
 import copy
 
-from StringIO import StringIO
+import six
 
 from xml4h.impls.interface import XmlImplAdapter
 from xml4h import nodes, exceptions
@@ -53,7 +53,13 @@ class ElementTreeAdapter(XmlImplAdapter):
     @classmethod
     def parse_string(cls, xml_str, ignore_whitespace_text_nodes=True):
         return cls.parse_file(
-            StringIO(xml_str),
+            six.StringIO(xml_str),
+            ignore_whitespace_text_nodes=ignore_whitespace_text_nodes)
+
+    @classmethod
+    def parse_bytes(cls, xml_bytes, ignore_whitespace_text_nodes=True):
+        return cls.parse_file(
+            six.BytesIO(xml_bytes),
             ignore_whitespace_text_nodes=ignore_whitespace_text_nodes)
 
     @classmethod
@@ -129,7 +135,8 @@ class ElementTreeAdapter(XmlImplAdapter):
         if isinstance(node, BaseET.Element):
             return True
         # For cElementTree we need to be more cunning (or find a better way)
-        if hasattr(node, 'makeelement') and isinstance(node.tag, basestring):
+        if hasattr(node, 'makeelement') \
+                and isinstance(node.tag, six.string_types):
             return True
 
     def map_node_to_class(self, node):
@@ -187,7 +194,7 @@ class ElementTreeAdapter(XmlImplAdapter):
             if n == node:
                 continue
             # Ignore non-Elements
-            if not isinstance(n.tag, basestring):
+            if not isinstance(n.tag, six.string_types):
                 continue
             if ns_uri != '*' and self.get_node_namespace_uri(n) != ns_uri:
                 continue
@@ -288,11 +295,13 @@ class ElementTreeAdapter(XmlImplAdapter):
 
     def get_node_name_prefix(self, node):
         # Ignore non-elements
-        if not isinstance(node.tag, basestring):
+        if not isinstance(node.tag, six.string_types):
             return None
-        # Believe nodes that know their own prefix (likely only ETAttribute)
-        if hasattr(node, 'prefix'):
-            return node.prefix
+        # Believe nodes that have their own prefix (likely only ETAttribute)
+        prefix = getattr(node, 'prefix', None)
+        if prefix:
+            return prefix
+        # Derive prefix by unpacking node name
         qname, ns_uri, prefix, local_name = self._unpack_name(node.tag, node)
         if prefix:
             # Don't add unnecessary excess namespace prefixes for elements
@@ -313,11 +322,16 @@ class ElementTreeAdapter(XmlImplAdapter):
             return target
         elif node.tag == BaseET.Comment:
             return node.text
-        else:
+        elif hasattr(node, 'value'):
             return node.value
+        else:
+            return node.text
 
     def set_node_value(self, node, value):
-        node.value = value
+        if hasattr(node, 'value'):
+            node.value = value
+        else:
+            self.set_node_text(node, value)
 
     def get_node_text(self, node):
         return node.text
@@ -328,11 +342,11 @@ class ElementTreeAdapter(XmlImplAdapter):
     def get_node_attributes(self, element, ns_uri=None):
         # TODO: Filter by ns_uri
         attribs_by_qname = {}
-        for n, v in element.attrib.items():
+        for n, v in list(element.attrib.items()):
             qname, ns_uri, prefix, local_name = self._unpack_name(n, element)
             attribs_by_qname[qname] = ETAttribute(
                 qname, ns_uri, prefix, local_name, v, element)
-        return attribs_by_qname.values()
+        return list(attribs_by_qname.values())
 
     def has_node_attribute(self, element, name, ns_uri=None):
         return name in [a.qname for a
@@ -469,7 +483,7 @@ class ElementTreeAdapter(XmlImplAdapter):
             # human-assigned prefix if available.
             curr_node = node
             while self._is_node_an_element(curr_node):
-                for n, v in curr_node.attrib.items():
+                for n, v in list(curr_node.attrib.items()):
                     if v == uri:
                         if n.startswith('xmlns:'):
                             result = n.split(':')[1]
